import numpy as np 
import pandas as pd
import json 
from utils import plotting

def vdv_generate_input():


    # TODO: absolute path
    with open ('config.json') as config_file:
        config = json.load(config_file)
    #print(config)
    
    # Create input sequence for simulation
    n = config['vdv_model']['n_iterations']
    

    #u = np.array([np.random.uniform(low=Fmin, high=Fmax, size=(n,1)), np.random.uniform(low=Qkmin, high=Qkmax, size=(n,1))])
    #u = np.concatenate(u, axis = 1)

    try: 
        options = config['vdv_model']['input_generation']['options']
        options['n'] = n
        print("Fetched options from model")
    except:
        # Default values
        options = {'n':n, 
        'input_interval_size':7200, 
        'perturbation_style':'single',
        'F_min':10, 
        'F_max':150, 
        'Fin_0':14.19, 
        'max_change_Fin': 10,
        'Qk_min': -8500, 
        'Qk_max':-500, 
        'Qk_0':-1113.5, 
        'max_change_Qk': 50}
    
    u = step_wise_perturbations(options)
    #u = uniform_within_bounds()
    #print(u)
    df = pd.DataFrame(data=u, columns = ['Fin', 'Qk'])
    #uk = ["F_in", "Q_k"] + u
    print("Generated:\n",df)

    df.to_csv(config['vdv_model']['input_generation']['storage_file'])

    #plotting.vdv_plot_input_sequence(u, dt=1/config['vdv_model']['simulation']['samples_per_hour'])

    # try: 
    #     new_u = pd.read_csv(config['vdv_model']['input_generation']['storage_file'], index_col=0)
    # except:
    #     new_u = np.zeros((10,1))

    # print("Read from file:\n",new_u)


def uniform_within_bounds(Fmin = 10, Fmax = 150, Qkmin = -8500, Qkmax = -500, n = 2500):
    u = np.array([np.random.uniform(low=Fmin, high=Fmax, size=(n,1)), np.random.uniform(low=Qkmin, high=Qkmax, size=(n,1))])
    u = np.concatenate(u, axis = 1)
    return u


def step_wise_perturbations(options):
    n = options['n']
    step_size = options['input_interval_size']
    perturbation = options['perturbation_style']
    
    Fmin = options['F_min']
    Fmax = options['F_max']
    try:
        Fin0 = options['Fin_0']
    except: 
        Fin0 = np.random.uniform(low=Fmin, high=Fmax)
    max_change_Fin = options['max_change_Fin']

    Qkmin = options['Qk_min']
    Qkmax = options['Qk_max']
    try:
        Qk0 = options['Qk_0']
    except:
        Qk0 = np.random.uniform(low=Qkmin, high=Qkmax)
    max_change_Qk = options['max_change_Qk']


    u = np.zeros((n,2))
    Fin_i = Fin0
    Qk_i = Qk0

    for i in range(0, n, step_size):
        if i + step_size <=n:
            u[i : i + step_size] = [[Fin_i, Qk_i]]*step_size

        if perturbation == 'double':
            Fin_i += np.random.uniform(low = -max_change_Fin, high = max_change_Fin)
            Qk_i += np.random.uniform(low = -max_change_Qk, high = max_change_Qk)
        elif perturbation == 'F_only':
            Fin_i += np.random.uniform(low = -max_change_Fin, high = max_change_Fin)
        elif perturbation == 'Qk_only':
            Qk_i += np.random.uniform(low = -max_change_Qk, high = max_change_Qk)
        elif np.random.randint(0,2) == 1: #if single, chose one at random
            Fin_i += np.random.uniform(low = -max_change_Fin, high = max_change_Fin)
        else: 
            Qk_i += np.random.uniform(low = -max_change_Qk, high = max_change_Qk)

        # Don't breach constraints
        if Fin_i > Fmax:
            Fin_i = Fmax
        elif Fin_i < Fmin:
            Fin_i = Fmin
        
        if Qk_i > Qkmax:
            Qk_i = Qkmax
        elif Qk_i < Qkmin:
            Qk_i = Qkmin


    # to assure n inputs:
    rest = n%step_size
    if rest > 0:
        u[n-rest:] = [[Fin_i, Qk_i]]*rest
    
    return u

    # Other than random uniform perturbations? Always max? Different distributions?


vdv_generate_input()

