import matplotlib.pyplot as plt
import pandas as pd
import torch
from torch.utils.data import DataLoader
from math import sqrt
from pathlib import Path
import numpy as np
import json

#local
from neural_net_class import NeuralNet as Net
from utils import plotting


## Get path to source directory
src_dir = Path(__file__).parent.parent.absolute()

with open (src_dir / 'config.json') as config_file:
    config = json.load(config_file)

dt = 1/config['vdv_model']['simulation']['samples_per_hour']

INPUT_COLS = config['neural_net']['input_cols']
OUTPUT_COLS = config['neural_net']['output_cols']

layers = [len(INPUT_COLS), 50, 50, len(OUTPUT_COLS)]
net = Net(layers)

## Load Net
#net.load(src_dir / "neural-nets/well_trained_nn")
net.load(src_dir / config["neural_net"]["trained_net_storage_file"])

## Load input data batch
df = pd.read_csv(src_dir / config["predictions"]["prediction_input_data"], index_col=0)
input_set = df
#df = pd.read_csv(src_dir / 'data/input_batch_preprocessed.csv', index_col=0)

# Produce time array 
time = np.linspace(0,(df.shape[0]-1)*60, df.shape[0])

# Get input and output as torch tensors
x_input = torch.from_numpy(input_set[INPUT_COLS].values).to(torch.float)

# Unused - theoretically unknown, but known due to simulation.
y_input = torch.from_numpy(input_set[OUTPUT_COLS].values).to(torch.float)

# Make prediction
predictions = net(x_input)
predictions = predictions.detach().numpy().T

# Visualization
input_sequence_val = input_set[['Fin', 'Qk']].values[:-1]

#plotting.plot_vdv(states=predictions, u= input_sequence_val, TF=input_sequence_val.shape[0]*dt, source='Neural Net')
#plotting.vdv_plot_states(states=predictions, source='Neural Net')

sources = {'Neural Net': predictions}
## Cheating: check predictions
sources = {'Neural Net': predictions, 'Real data': y_input.detach().numpy().T}

plotting.vdv_plot_states_cmp(sources, time=time, output_cols=OUTPUT_COLS)

df_time = pd.read_csv(src_dir / 'data/vdv_steady_state_time.csv', index_col = 0)
time = np.concatenate(df_time.values, axis = 0)

# Separated steady state periods:
if np.argwhere(np.diff(time)>1.9*60*dt).shape[0] > 0:
    #print(np.argwhere(np.diff(time)>(60+1)/config['vdv_model']['simulation']['samples_per_hour']).shape)
    periods = np.split(np.arange(0, time.shape[0], 1), np.concatenate(np.argwhere(np.diff(time)>1.9*60*dt)+1))
    #periods = np.split(np.arange(0, time.shape[0], 1), np.concatenate(np.argwhere(np.diff(time)>0.017)+1))
    
    print("Number of periods: ", len(periods))
    plotting.vdv_plot_states_cmp_period_sep(sources, time=time, output_cols=OUTPUT_COLS, periods=periods)

