from casadi import *
import numpy as np
import math
import matplotlib.pyplot as plt
import pandas as pd
from pathlib import Path
import json

from utils import plotting


with open ('config.json') as config_file:
    config = json.load(config_file)


#raw_dataset_file = "data/vdv_dataset.csv"
#steady_state_dataset_file = "data/vdv_steady_state_dataset.csv"
raw_dataset_file = config["vdv_model"]["data_extraction"]["raw_data_file"]
steady_state_dataset_file = config["vdv_model"]["data_extraction"]["steady_state_data_storage_file"]

steady_state_variable = config['vdv_model']['data_extraction']['steady_state_variable']
state_threshold = config['vdv_model']['data_extraction']['state_change_threshold']
period_threshold = config['vdv_model']['data_extraction']['steady_period_duration_threshold']
dt = 1/config['vdv_model']['simulation']['samples_per_hour']


units = {'Tout': 'C', 'Tkout': 'C', 'CAOut': 'Mol/l', 'CBout': 'Mol/l', 'CCOut': 'Mol/l', 'CDOut': 'Mol/l'}

def extract_data(steady_var = 'Tout', dt = 1/3600, state_threshold = 0.005, period_threshold = 120):
    # Steady periods contains all datapoints of the steady periods
    #steady_period = []
    #steady_periods = []
    steady_period_index = []
    steady_periods_indices = []
    n_periods = 0
    
    df = pd.read_csv(raw_dataset_file, index_col=0)
    raw_data = df.to_numpy()
    output_values = df[steady_var].values
    # TODO: Generalize to not only have temperature
    # Temperature value of first datapoint
    S = output_values[0]
    print("Inital value for ", steady_var, ": ", S)
    for i in range(df.shape[0]):
        if np.abs(S - output_values[i]) < state_threshold:
            #steady_period.append(raw_data[i])
            steady_period_index.append(i)
        else:
            if len(steady_period_index) > period_threshold:
                n_periods+=1
                #steady_periods.append(steady_period)
                #steady_period = []
                steady_periods_indices += steady_period_index
            # When to change setpoint?
            S = output_values[i]
            steady_period_index = []
    
    if len(steady_period_index) > 0:
        #steady_periods.append(steady_period)
        n_periods+=1
        steady_periods_indices += steady_period_index
    
    print("Steady state variable: ", steady_var)
    print("Steady state change threshold = ", state_threshold,' ', units[steady_var])
    print("Total number of datapoints in steady state: ", len(steady_periods_indices))
    print("Steady period duration threshold = ", period_threshold, " datapoints.")
    print("Number of steady state periods: ", n_periods)


    # Mock to test effect of split periods
    #steady_periods_indices = np.concatenate((np.arange(0,6000,1), np.arange(6000,7000,1), np.arange(11000,12000,1), np.arange(13300,14300,1)))
    time = np.linspace(0, (df.shape[0]-1)*60*dt, df.shape[0])
 
    try:
        extracted_data = raw_data[np.array(steady_periods_indices)]
        extracted_time = time[np.array(steady_periods_indices)]
    except: 
        extracted_data = []
        extracted_time = []
    
    df_steady_state_time = pd.DataFrame(data = extracted_time, columns = ['Time'])
    df_steady_state_dataset = pd.DataFrame(data = extracted_data, columns = ['CAin', 'CBin', 'CCin', 'CDin', 'Tin', 'Tkin', 'Fin', 'Qk', 'CAout', 'CBout', 'CCout', 'CDout', 'Tout','Tkout'])

    df_steady_state_time.to_csv("data/vdv_steady_state_time.csv")
    df_steady_state_dataset.to_csv(steady_state_dataset_file)
    
    #print extracted data

    sources = {'Extracted data': extracted_data.T}
    # Separated steady state periods:
    if np.argwhere(np.diff(extracted_time)>1.9*60*dt).shape[0] > 0:
        #print(np.argwhere(np.diff(time)>(60+1)/config['vdv_model']['simulation']['samples_per_hour']).shape)
        periods = np.split(np.arange(0, extracted_time.shape[0], 1), np.concatenate(np.argwhere(np.diff(extracted_time)>1.9*60*dt)+1))
        #periods = np.split(np.arange(0, time.shape[0], 1), np.concatenate(np.argwhere(np.diff(time)>0.017)+1))
        
        print("Number of periods: ", len(periods))
        plotting.vdv_plot_states_cmp_period_sep(sources, time=extracted_time, output_cols=config["neural_net"]["all_output_cols"], periods=periods)
        #plotting.vdv_plot_states_cmp_period_sep(sources, time=extracted_time, output_cols=['Tout','Tkout'], periods=periods)

    #print(extracted_data.shape)
    
    # with open ('data/steady_state_indices.json', 'w') as outfile:
    #     json.dump(steady_periods_indices, outfile)

    
    #df_steady_state_period = pd.DataFrame(data=np.array(steady_periods_indices), columns = ["Indices"])
    #df_steady_state_period.to_csv("data/vdv_steady_state_indices.csv")
    

    # Test of extracting properly
    # src_dir = Path(__file__).parent.parent.absolute()

    # read_indices = pd.read_csv(src_dir / 'data/vdv_steady_state_indices.csv', index_col = 0)
    # print(np.concatenate(read_indices.values, axis=0))
    # st_index = np.concatenate(read_indices.values, axis=0)
    
    # extracted_data = raw_data[np.array(st_index)]
    # print(extracted_data.shape)



extract_data(steady_var=steady_state_variable, dt=dt, state_threshold=state_threshold, period_threshold=period_threshold)