import matplotlib.pyplot as plt
import pandas as pd
import torch
from torch.utils.data import DataLoader
from math import sqrt
from pathlib import Path
import numpy as np
import json

#local
from neural_net_class import NeuralNet as Net
from utils import plotting




## Get path to source directory
src_dir = Path(__file__).parent.parent.absolute()

# TODO: absolute path
with open (src_dir / 'config.json') as config_file:
    config = json.load(config_file)

dt = 1/config['vdv_model']['simulation']['samples_per_hour']

#INPUT_COLS = ['CAin', 'CBin', 'CCin', 'CDin', 'Tin', 'Tkin','Fin', 'Qk']
#OUTPUT_COLS = ['CAout', 'CBout', 'CCout', 'CDout', 'Tout', 'Tkout']
INPUT_COLS = config['neural_net']['input_cols']
OUTPUT_COLS = config['neural_net']['output_cols']

layers = [len(INPUT_COLS), 50, 50, len(OUTPUT_COLS)]
net = Net(layers)

## Load Net
#net.load(src_dir / "neural-nets/trained_nn")
net.load(src_dir / config["neural_net"]["trained_net_storage_file"])

## Load dataset and define test_set
#df = pd.read_csv(src_dir / 'data/vdv_steady_state_dataset.csv', index_col=0)
try: 
    df = pd.read_csv(src_dir / config["neural_net"]["data_set"], index_col=0)
except:
    print("Unable to load dataset")

low_index = int(df.shape[0]*0.65)
up_index = int(df.shape[0]*0.95)
test_set = df.iloc[low_index:up_index]

# Fetch time array to plot against
df_time = pd.read_csv(src_dir / 'data/vdv_steady_state_time.csv', index_col = 0)
time = np.concatenate(df_time.values, axis = 0)
time = time[low_index:up_index]

# # Evaluate the model on test data

# Get input and output as torch tensors
x_test = torch.from_numpy(test_set[INPUT_COLS].values).to(torch.float)
y_test = torch.from_numpy(test_set[OUTPUT_COLS].values).to(torch.float)

# Make prediction
pred_test = net(x_test)

# Visualization
input_sequence_val = test_set[['Fin', 'Qk']].values[:-1]

#plot_vdv(states=pred_test.detach().numpy().T, u= input_sequence_val, TF=input_sequence_val.shape[0]*dt, source='Neural Net')
#plotting.vdv_plot_states(states=pred_test.detach().numpy().T, source='Neural Net')
#plotting.vdv_plot_states(states=y_test.detach().numpy().T, source = 'Model')

# All datapoints
#sources = {'Neural Net': pred_test.detach().numpy().T, 'Model': y_test.detach().numpy().T}

# Only datapoints with steady state

ss_predictions = pred_test.detach().numpy().T
ss_measurements = y_test.detach().numpy().T
sources = {'Neural Net': ss_predictions, 'Model': ss_measurements}

plotting.vdv_plot_states_cmp(sources, time=time, output_cols=OUTPUT_COLS)

# Separated steady state periods:

if np.argwhere(np.diff(time)>1.9*60*dt).shape[0] > 0:
    #print(np.argwhere(np.diff(time)>(60+1)/config['vdv_model']['simulation']['samples_per_hour']).shape)
    periods = np.split(np.arange(0, time.shape[0], 1), np.concatenate(np.argwhere(np.diff(time)>1.9*60*dt)+1))
    #periods = np.split(np.arange(0, time.shape[0], 1), np.concatenate(np.argwhere(np.diff(time)>0.017)+1))
    
    print("Number of periods: ", len(periods))
    plotting.vdv_plot_states_cmp_period_sep(sources, time=time, output_cols=OUTPUT_COLS, periods=periods)


# Compute MSE, MAE and MAPE on test data
print('Error on test data')

mse_test = torch.mean(torch.pow(pred_test - y_test, 2))
print(f'MSE: {mse_test.item()}')

mae_test = torch.mean(torch.abs(pred_test - y_test))
print(f'MAE: {mae_test.item()}')

mape_test = 100*torch.mean(torch.abs(torch.div(pred_test - y_test, y_test)))
print(f'MAPE: {mape_test.item()} %')

