import matplotlib.pyplot as plt 
import numpy as np




def plot_vdv(states, u, TF, source):
    n = u.shape[0]
    time = np.linspace(0, TF, n + 1)

    # Concentrations
    fig1 = plt.figure(1)
    plt.title(source+": Concentration of species in CSTR")
    plt.plot(time, states[0].T, label = 'species A')
    plt.plot(time, states[1].T, label = 'species B')
    plt.plot(time, states[2].T, label = 'species C')
    plt.plot(time, states[3].T, label = 'species D')
    plt.xlabel("Time [min]")
    plt.ylabel("Concentration [mol/l]")
    plt.legend()

    # Temperatures
    fig2 = plt.figure(2)
    plt.title(source+ ":Temperature")
    plt.plot(time, states[4].T, label = 'Temperature in CSTR')
    plt.plot(time, states[5].T, label = 'Temperature of cooling jacket')
    plt.xlabel("Time [min]")
    plt.ylabel("Temperature [C]")
    plt.legend()

    # Input variables
    fig3 = plt.figure(3, figsize=(12,5))
    ax1 = fig3.add_subplot(1,2,1)
    ax2 = fig3.add_subplot(1,2,2)
    fig3.suptitle(source+ ": Input variables")
    ax1.plot(time[:-1], u[:,0].T, label = 'Feed rate F')
    ax2.plot(time[:-1], u[:,1].T, label = 'Jacket cooling rate Qk')
    ax1.set_xlabel('Time [min]')
    ax2.set_xlabel('Time [min]')
    ax1.set_ylabel('Feed rate [l/hr]')
    ax2.set_ylabel('Jacket cooling rate [kJ/hr]')
    ax1.legend()
    ax2.legend()
    
    plt.show()

def vdv_plot_states(states, source, dt=1/3600):
    n = states.shape[1]
    time = np.linspace(0, (n-1)*dt*60, n)

    fig1 = plt.figure(1)
    ax1 = fig1.add_subplot(1,1,1)
    fig1.suptitle(source+": Concentration of species in CSTR")
    for i in range(4):
        ax1.plot(time, states[i].T, label = 'species '+chr(i + ord('A')))
    ax1.set_xlabel("Time [min]")
    ax1.set_ylabel("Concentration [mol/l]")
    ax1.legend()

    fig2 = plt.figure(2)
    ax21 = fig2.add_subplot(1,1,1)
    fig2.suptitle(source+": Temperature")
    ax21.plot(time, states[4].T, label = 'Temperature in CSTR')
    ax21.plot(time, states[5].T, label = 'Temperature of cooling jacket')
    ax21.set_xlabel("Time [min]")
    ax21.set_ylabel("Temperature [C]")
    ax21.legend()
    

    plt.show()
    return fig1, fig2



def vdv_plot_states_cmp(sources, time, output_cols, dt=1/3600):
    plot_style = ':'
    c_colors = ['r', 'b', 'g', 'y']
    i = 0

    if 'CAout' in output_cols or 'CBout' in output_cols or 'CCout' in output_cols or 'CDout' in output_cols:
        fig1 = plt.figure(1)
        ax1 = fig1.add_subplot(1,1,1)
        fig1.suptitle("Concentration of species in CSTR")
        for (source, states) in sources.items():
            c_iter = iter(output_cols)
            c_label = next(c_iter)
            ci = i
            while c_label[0] == 'C':
                ax1.plot(time, states[ci].T, plot_style, label= source+' species ' + c_label)
                ci+=1
                try: 
                    c_label = next(c_iter)
                except:
                    break
            #for i in range(4):
            #    ax1.plot(time, states[i].T, plot_style, label= source+' species '+chr(i + ord('A')))
            plot_style = '-'
        i = ci
        ax1.set_xlabel("Time [min]")
        ax1.set_ylabel("Concentration [mol/l]")
        ax1.legend()
        plt.show()
    
    t_colors = ['r', 'b']
    plot_style = ':'
    # if 'Tout' in output_cols or 'Tkout' in output_cols:
    #     fig2 = plt.figure(2, figsize=(9,4))
    #     fig2.suptitle("Temperature")

    if 'Tout' in output_cols and 'Tkout' in output_cols:
        fig2 = plt.figure(2, figsize=(9,4))
        fig2.suptitle("Temperature")
        ax21 = fig2.add_subplot(1,2,1)
        ax22 = fig2.add_subplot(1,2,2)
    elif 'Tout' in output_cols:
        fig2 = plt.figure(2)
        fig2.suptitle("Temperature")
        ax21 = fig2.add_subplot(1,1,1)
    elif 'Tkout' in output_cols:
        fig2 = plt.figure(2)
        fig2.suptitle("Temperature")
        ax22 = fig2.add_subplot(1,1,1)

    for (source, states) in sources.items():
        t_iter = iter(output_cols) #Can quicken this by continuing from c_label. Unknown if in scope
        t_label = next(t_iter)
        ti = i
        while t_label[0] == 'T':
            if t_label == 'Tout':
                ax21.plot(time, states[ti].T, plot_style, label = source+': Temperature in CSTR')
            elif t_label == 'Tkout':
                ax22.plot(time, states[ti].T, plot_style, label = source+': Temperature of cooling jacket')
            ti+=1
            try: 
                t_label = next(t_iter)
            except:
                break
        # ax21.plot(time, states[4].T, plot_style, label = source+': Temperature in CSTR')
        # ax22.plot(time, states[5].T, plot_style, label = source+': Temperature of cooling jacket')
        plot_style = '-'
    if 'Tout' in output_cols:
        ax21.set_xlabel("Time [min]")
        ax21.set_ylabel("Temperature [C]")
        ax21.legend()
    if 'Tkout' in output_cols:
        ax22.set_xlabel("Time [min]")
        ax22.set_ylabel("Temperature [C]")
        ax22.legend()
    if 'Tout' in output_cols or 'Tkout' in output_cols:
        plt.show()
    #return fig1, fig2


def vdv_plot_states_cmp_period_sep(sources, time, output_cols, periods, dt=1/3600):
    #n = sources['Neural Net'].shape[1]
    plot_style = ':'
    c_colors = ['b', 'y', 'g', 'r']
    i = 0

    if 'CAout' in output_cols or 'CBout' in output_cols or 'CCout' in output_cols or 'CDout' in output_cols:
        fig1 = plt.figure(1)
        ax1 = fig1.add_subplot(1,1,1)
        fig1.suptitle("Concentration of species in CSTR")
        for (source, states) in sources.items():
            c_iter = iter(output_cols)
            c_label = next(c_iter)
            ci = i
            labeled = {'CAout': False, 'CBout': False, 'CCout': False, 'CDout':False}
            while c_label[0] == 'C':
                for p in periods: 
                    if labeled[c_label]:
                        ax1.plot(time[p], states[ci][p].T, plot_style, color=c_colors[ci])
                    else:
                        ax1.plot(time[p], states[ci][p].T, plot_style, color=c_colors[ci], label = source + ': '+c_label)
                        labeled[c_label] = True
                ci+=1
                try: 
                    c_label = next(c_iter)
                except:
                    break
                #ax1.set_label(source+' species '+chr(i + ord('A')))
            plot_style = '-'
        i = ci
        ax1.set_xlabel("Time [min]")
        ax1.set_ylabel("Concentration [mol/l]")
        ax1.legend(loc='best', ncol=2)
        plt.show()
    
    #t_colors = ['r', 'b']
    plot_style = [':b', ':y']
    if 'Tout' in output_cols or 'Tkout' in output_cols:
        fig2 = plt.figure(2)
        fig2.suptitle("Temperature")

    if 'Tout' in output_cols and 'Tkout' in output_cols:
        #ax21 = fig2.add_subplot(1,2,1)
        #ax22 = fig2.add_subplot(1,2,2)
        ax21 = fig2.add_subplot(1,1,1)
    elif 'Tout' in output_cols:
        ax21 = fig2.add_subplot(1,1,1)
    elif 'Tkout' in output_cols:
        #ax22 = fig2.add_subplot(1,1,1)
        ax21 = fig2.add_subplot(1,1,1)

    for (source, states) in sources.items():
        t_labeled = False
        tk_labeled = False
        t_iter = iter(output_cols) #Can quicken this by continuing from c_label. Unknown if in scope
        t_label = next(t_iter)
        ti = i
        #ti = output_cols.index(t_label)
        while t_label[0] != 'T':
            try: 
                t_label = next(t_iter)
            except:
                break
        while t_label[0] == 'T':
            if t_label == 'Tout':
                for p in periods:
                    if t_labeled:
                        ax21.plot(time[p], states[ti][p].T, plot_style[ti-i])
                    else:
                        ax21.plot(time[p], states[ti][p].T, plot_style[ti-i], label=source + ': '+ t_label)
                        t_labeled = True
            elif t_label == 'Tkout':
                for p in periods:
                    if tk_labeled:
                        ax21.plot(time[p], states[ti][p].T, plot_style[ti-i])
                    else:
                        ax21.plot(time[p], states[ti][p].T, plot_style[ti-i], label=source + ': ' + t_label)
                        tk_labeled = True
            ti+=1
            try: 
                t_label = next(t_iter)
            except:
                break
        plot_style = ['-c','-y'] #only supports two simultanous sources
    
    if 'Tout' in output_cols:
        ax21.set_xlabel("Time [min]")
        ax21.set_ylabel("Temperature [C]")
        ax21.legend()
    if 'Tkout' in output_cols:
        ax21.set_xlabel("Time [min]")
        ax21.set_ylabel("Temperature [C]")
        ax21.legend()
    if 'Tout' in output_cols or 'Tkout' in output_cols:
        #plt.legend()
        plt.show()
    #return fig1, fig2

def vdv_plot_input_sequence(u, dt = 1/3600):    
    n = u.shape[0]
    time = np.linspace(0, (n-1)*dt*60, n + 1)

    # Input variables
    fig1 = plt.figure(1, figsize=(12,5))
    ax1 = fig1.add_subplot(1,2,1)
    ax2 = fig1.add_subplot(1,2,2)
    fig1.suptitle("Input variables")
    ax1.plot(time[:-1], u[:,0].T, label = 'Feed rate F')
    ax2.plot(time[:-1], u[:,1].T, label = 'Jacket cooling rate Qk')
    ax1.set_xlabel('Time [min]')
    ax2.set_xlabel('Time [min]')
    ax1.set_ylabel('Feed rate [l/hr]')
    ax2.set_ylabel('Jacket cooling rate [kJ/hr]')
    ax1.legend()
    ax2.legend()
    
    plt.show()
    return fig1